// *
// * Project name:
//     PROGRAMMING FROM UART INTERNAL RTCC AND USE
// * Copyright:
//     (c) Teolab.it
// * Revision History:
//     20220518:
//       - initial release (FJ);
// * Description:
//     A simple program to use the PIC18F87K22 internal RTCC, programming 
//     the parameters from UART with a simple 14 characters string.
// * Test configuration:
//     MCU:             PIC18F87K22
//                      https://www.microchip.com/en-us/product/PIC18F87K22
//     Dev.Board:       EasyPIC-PRO-V7
//                      https://www.mikroe.com/easypic-pro-v7
//     Oscillator:      16.0000 MHz Crystal
//     Ext. Modules:    GLCD/MCU
//     SW:              mikroC PRO for PIC
//                      https://www.mikroe.com/mikroc-pic
// * NOTES:
//     - Turn on SW4.2 - SW5.1 - SW5.2.
//     - Install GLCD on slot
//     - Install PIC18F87K22 MCU on socket
//     - For write new data and time, send from PC via UART this string:
//       HhhmmssWddmmyy -> W = weekday -> 0=Sun/6=Sat.
//       H1615003180522 -> 16:15:00 Wednesday=3 18/05/22
//     - At power up the RTCC is stopped and the GLCD may display meaningless 
//       numbers. Send the programming string from UART to start the clock 
//       correctly.
// *

#include "built_in.h"
#include <stdint.h>
#include <string.h>
#include "OBJECTS.h"

char GLCD_DataPort at PORTD;

sbit GLCD_CS1 at LATB0_bit;
sbit GLCD_CS2 at LATB1_bit;
sbit GLCD_RS  at LATB2_bit;
sbit GLCD_RW  at LATB3_bit;
sbit GLCD_EN  at LATB4_bit;
sbit GLCD_RST at LATB5_bit;

sbit GLCD_CS1_Direction at TRISB0_bit;
sbit GLCD_CS2_Direction at TRISB1_bit;
sbit GLCD_RS_Direction  at TRISB2_bit;
sbit GLCD_RW_Direction  at TRISB3_bit;
sbit GLCD_EN_Direction  at TRISB4_bit;
sbit GLCD_RST_Direction at TRISB5_bit;

sbit LED_SYS at LATJ3_bit;
sbit LED_SYS_Direction at TRISJ3_bit;

sbit LED1_UART at LATJ2_bit;
sbit LED1_UART_Direction at TRISJ2_bit;

sbit LED2_UART at LATJ1_bit;
sbit LED2_UART_Direction at TRISJ1_bit;

int Txt[14] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
char I,Ready,TempTime;
unsigned long Res;
unsigned int year,day,month,weekday,hours,seconds,minutes,secondsOld;
char SetSec,SetMin,SetHr,SetWeek,SetDay,SetMn,SetYear;
char TXT0[10],TXT1[10],TXT2[10],TXT3[10],TXT4[10],TXT5[10];
char TXT7[10],TXT8[10],TXT9[10],TXT10[10],TXT11[10];
char *TXT6;

void FlashLed() {
  LED_SYS = ~(LED_SYS);
  Delay01s();
}

void interrupt() {
  if (RCIF_bit == 1) {
    Txt[I] = UART1_Read();
    I++;
    
    if (I == 14) {
        I = 0;
        Ready = 1;
    }
    
    RCIF_bit = 0;
  }
}

void Zero_Fill(char *value) {
  if (value[1] == 0) {
    value[1] = value[0];
    value[0] = 48;
    value[2] = 0;
  }
}

void VisualTime() {
 ByteToStr(hours,TXT0);
 Ltrim(TXT0);
 Zero_Fill(TXT0);

 ByteToStr(minutes,TXT1);
 Ltrim(TXT1);
 Zero_Fill(TXT1);

 ByteToStr(seconds,TXT2);
 Ltrim(TXT2);
 Zero_Fill(TXT2);
 
 ByteToStr(Day,TXT3);
 Ltrim(TXT3);
 Zero_Fill(TXT3);
 
 ByteToStr(Month,TXT4);
 Ltrim(TXT4);
 Zero_Fill(TXT4);
 
 ByteToStr(Year,TXT5);
 Ltrim(TXT5);
 Zero_Fill(TXT5);

 Glcd_Write_Text(TXT0,0,2,1);
 Glcd_Write_Text(":",12,2,1);
 Glcd_Write_Text(TXT1,19,2,1);
 Glcd_Write_Text(":",32,2,1);
 Glcd_Write_Text(TXT2,38,2,1);
 
 Glcd_Write_Text(TXT3,0,3,1);
 Glcd_Write_Text("/",12,3,1);
 Glcd_Write_Text(TXT4,19,3,1);
 Glcd_Write_Text("/",32,3,1);
 Glcd_Write_Text(TXT5,38,3,1);

 switch (weekday) {
   case 0: 
   TXT6 = "Sunday";
   break;
   
   case 1: 
   TXT6 = "Monday"; 
   break;
   
   case 2: 
   TXT6 = "Tuesday"; 
   break;
   
   case 3: 
   TXT6 = "Wednesday"; 
   break;
   
   case 4: 
   TXT6 = "Thursday"; 
   break;
   
   case 5: 
   TXT6 = "Friday"; 
   break;
   
   case 6: 
   TXT6 = "Saturday"; 
   break;
 }
 
 Glcd_Write_Text(TXT6,0,4,1);
}

void main() {
 ANCON0 = 0;
 ANCON1 = 0;
 ANCON2 = 0;
 CM1CON = 0;
 CM2CON = 0;
 CM3CON = 0;
 LED_SYS_Direction = 0;
 LED1_UART_Direction = 0;
 LED2_UART_Direction = 0;
 
 LED1_UART = 0;
 LED2_UART = 0;
 LED_SYS = 0;
 
 //RTCC_Init();
 
 Glcd_Init();
 Glcd_Fill(0x00);
 Glcd_Write_Text("18F87K22-INSIDE-RTCC:",0,0,1);
 Glcd_H_Line(0,127,11,1);
 Glcd_H_Line(0,127,45,1);
 Glcd_Write_Text("www.teolab.it",20,6,1);
 
 UART1_Init(9600);
 Delay01s();
 I = 0;
 
 RC1IE_bit = 1;
 RC1IF_bit = 0;
 PEIE_bit  = 1;
 GIE_bit   = 1;
 
 while (1) {
   FlashLed();
   Read_Internal_RTCC();
   VisualTime();
   
   if (Ready == 1) {
       UART1_Write(Txt[0]);
       UART1_Write(Txt[1]);
       UART1_Write(Txt[2]);
       UART1_Write(Txt[3]);
       UART1_Write(Txt[4]);
       UART1_Write(Txt[5]);
       UART1_Write(Txt[6]);
       
       UART1_Write(Txt[7]);
       UART1_Write(Txt[8]);
       UART1_Write(Txt[9]);
       UART1_Write(Txt[10]);
       UART1_Write(Txt[11]);
       UART1_Write(Txt[12]);
       UART1_Write(Txt[13]);
   
       if (Txt[0] == 'H') {
           LED1_UART = 1;
           Delay01s();
           LED1_UART = 0;
           
           TempTime = ((Txt[1] - 48) * 10) + (Txt[2] - 48);
           SetHr = dec2bcd(TempTime);

           TempTime = ((Txt[3] - 48) * 10) + (Txt[4] - 48);
           SetMin = dec2bcd(TempTime);

           TempTime = ((Txt[5] - 48) * 10) + (Txt[6] - 48);
           SetSec = dec2bcd(TempTime);
           
           TempTime = (Txt[7] - 48);
           SetWeek = dec2bcd(TempTime);
           
           TempTime = ((Txt[8] - 48) * 10) + (Txt[9] - 48);
           SetDay = dec2bcd(TempTime);
           
           TempTime = ((Txt[10] - 48) * 10) + (Txt[11] - 48);
           SetMn = dec2bcd(TempTime);
           
           TempTime = ((Txt[12] - 48) * 10) + (Txt[13] - 48);
           SetYear = dec2bcd(TempTime);
           
           RTCC_Program();
       }
       
       Ready = 0;
   }
 }
}